@@ -1,7 +1,7 @@
 /*
  * gdkscreen.h
- * 
- * Copyright 2001 Sun Microsystems Inc. 
+ *
+ * Copyright 2001 Sun Microsystems Inc.
  *
  * Erwann Chenede <erwann.chenede@sun.com>
  *
@@ -34,47 +34,16 @@
 
 G_BEGIN_DECLS
 
-typedef struct _GdkScreenClass GdkScreenClass;
-
 #define GDK_TYPE_SCREEN            (gdk_screen_get_type ())
 #define GDK_SCREEN(object)         (G_TYPE_CHECK_INSTANCE_CAST ((object), GDK_TYPE_SCREEN, GdkScreen))
-#define GDK_SCREEN_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GDK_TYPE_SCREEN, GdkScreenClass))
 #define GDK_IS_SCREEN(object)      (G_TYPE_CHECK_INSTANCE_TYPE ((object), GDK_TYPE_SCREEN))
-#define GDK_IS_SCREEN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GDK_TYPE_SCREEN))
-#define GDK_SCREEN_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GDK_TYPE_SCREEN, GdkScreenClass))
 
-/**
- * GdkScreen:
- *
- * This is a currently just a placeholder typedef for the first argument of
- * the #GdkPointerHooks.window_at_pointer function in #GdkPointerHooks.
- * It will be used  when GDK gets multihead support.
- *
- * Since: 2.2
- */
-struct _GdkScreen
-{
-  GObject parent_instance;
-
-  guint GSEAL (closed) : 1;
-
-  cairo_font_options_t *GSEAL (font_options);
-  double GSEAL (resolution);	/* pixels/points scale factor for fonts */
-};
-
-struct _GdkScreenClass
-{
-  GObjectClass parent_class;
-
-  void (*size_changed) (GdkScreen *screen);
-  void (*composited_changed) (GdkScreen *screen);
-  void (*monitors_changed) (GdkScreen *screen);
-};
 
 GType        gdk_screen_get_type              (void) G_GNUC_CONST;
-GdkVisual*   gdk_screen_get_system_visual     (GdkScreen   *screen);
+
+GdkVisual *  gdk_screen_get_system_visual     (GdkScreen   *screen);
 GdkVisual *  gdk_screen_get_rgba_visual       (GdkScreen   *screen);
-gboolean     gdk_screen_is_composited	      (GdkScreen   *screen);
+gboolean     gdk_screen_is_composited         (GdkScreen   *screen);
 
 GdkWindow *  gdk_screen_get_root_window       (GdkScreen   *screen);
 GdkDisplay * gdk_screen_get_display           (GdkScreen   *screen);
@@ -91,13 +60,13 @@ gchar *      gdk_screen_make_display_name     (GdkScreen   *screen);
 gint          gdk_screen_get_n_monitors        (GdkScreen *screen);
 gint          gdk_screen_get_primary_monitor   (GdkScreen *screen);
 void          gdk_screen_get_monitor_geometry  (GdkScreen *screen,
-						gint       monitor_num,
-						GdkRectangle *dest);
+                                                gint       monitor_num,
+                                                GdkRectangle *dest);
 gint          gdk_screen_get_monitor_at_point  (GdkScreen *screen,
-						gint       x,
-						gint       y);
+                                                gint       x,
+                                                gint       y);
 gint          gdk_screen_get_monitor_at_window (GdkScreen *screen,
-						GdkWindow *window);
+                                                GdkWindow *window);
 gint          gdk_screen_get_monitor_width_mm  (GdkScreen *screen,
                                                 gint       monitor_num);
 gint          gdk_screen_get_monitor_height_mm (GdkScreen *screen,
@@ -106,20 +75,20 @@ gchar *       gdk_screen_get_monitor_plug_name (GdkScreen *screen,
                                                 gint       monitor_num);
 
 void          gdk_screen_broadcast_client_message  (GdkScreen       *screen,
-						    GdkEvent        *event);
+                                                    GdkEvent        *event);
 
 GdkScreen *gdk_screen_get_default (void);
 
 gboolean   gdk_screen_get_setting (GdkScreen   *screen,
-				   const gchar *name,
-				   GValue      *value);
+                                   const gchar *name,
+                                   GValue      *value);
 
 void                        gdk_screen_set_font_options (GdkScreen                  *screen,
-							 const cairo_font_options_t *options);
+                                                         const cairo_font_options_t *options);
 const cairo_font_options_t *gdk_screen_get_font_options (GdkScreen                  *screen);
 
 void    gdk_screen_set_resolution (GdkScreen *screen,
-				   gdouble    dpi);
+                                   gdouble    dpi);
 gdouble gdk_screen_get_resolution (GdkScreen *screen);
 
 GdkWindow *gdk_screen_get_active_window (GdkScreen *screen);
@@ -127,4 +96,4 @@ GList     *gdk_screen_get_window_stack  (GdkScreen *screen);
 
 G_END_DECLS
 
-#endif				/* __GDK_SCREEN_H__ */
+#endif  /* __GDK_SCREEN_H__ */
