@@ -8,7 +8,7 @@
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
@@ -60,7 +60,7 @@ find_name (const gchar *name)
   for (i = 0; i < n_names; i++)
     {
       if (strcmp (names[i].s, name) == 0)
-	return names[i].offset;
+        return names[i].offset;
     }
 
   fprintf (stderr, "BOO! %s not found\n", name);
@@ -73,8 +73,8 @@ find_name (const gchar *name)
 
 static gboolean
 parse_media_size (const gchar *size,
-		  gdouble     *width_mm, 
-		  gdouble     *height_mm)
+                  gdouble     *width_mm, 
+                  gdouble     *height_mm)
 {
   const gchar *p;
   gchar *e;
@@ -127,7 +127,7 @@ main (int argc, char *argv[])
       add_name (standard_names[i].name);
       add_name (standard_names[i].display_name);
       if (standard_names[i].ppd_name)
-	add_name (standard_names[i].ppd_name);
+        add_name (standard_names[i].ppd_name);
     }
 
   for (i = 0; i < n_extra; i++)
@@ -140,102 +140,101 @@ main (int argc, char *argv[])
   for (i = 0; i < n_names; i++)
     for (j = 0; j < n_names; j++)
       {
-	if (i == j) continue;
-	
-	if (names[i].len < names[j].len ||
-	    (names[i].len == names[j].len && j < i))
-	  {
-	    if (strcmp (names[i].s, names[j].s + names[j].len - names[i].len) == 0)
-	      {
-		names[i].suffix = j;
-		break;
-	      }
-	  }
+        if (i == j) continue;
+
+        if (names[i].len < names[j].len ||
+            (names[i].len == names[j].len && j < i))
+          {
+            if (strcmp (names[i].s, names[j].s + names[j].len - names[i].len) == 0)
+              {
+                names[i].suffix = j;
+                break;
+              }
+          }
       }
-  
+
   /* calculate offsets for regular entries */
   offset = 0;
   for (i = 0; i < n_names; i++)
     {
       if (names[i].suffix == -1)
-	{
-	  names[i].offset = offset;
-	  offset += names[i].len + 1;
-	}
+        {
+          names[i].offset = offset;
+          offset += names[i].len + 1;
+        }
     }
-  
+
   /* calculate offsets for suffixes */
   for (i = 0; i < n_names; i++)
     {
       if (names[i].suffix != -1)
-	{
-	  j = i;
-	  do {
-	    j = names[j].suffix;
-	  } while (names[j].suffix != -1);
-	  names[i].offset = names[j].offset + names[j].len - names[i].len;
-	}
+        {
+          j = i;
+          do {
+            j = names[j].suffix;
+          } while (names[j].suffix != -1);
+          names[i].offset = names[j].offset + names[j].len - names[i].len;
+        }
     }
-    
+
   printf ("/* Generated by gen-paper-names */\n\n");
 
   /* write N_ marked names */
 
   printf ("#if 0\n");
   for (i = 0; i < n_infos; i++)
-    printf ("/* translators, strip everything up to the first | */\n"
-	    "N_(\"paper size|%s\")\n", standard_names[i].display_name);
+    printf ("NC_(\"paper size\", \"%s\")\n", standard_names[i].display_name);
   printf ("#endif\n\n");
 
   /* write strings */
-  printf ("const char _gtk_paper_names[] =");
+  printf ("static const char paper_names[] =");
   for (i = 0; i < n_names; i++)
     {
       if (names[i].suffix == -1)
-	printf ("\n  \"%s\\0\"", names[i].s);
+        printf ("\n  \"%s\\0\"", names[i].s);
     }
   printf (";\n\n");
 
   /* dump PaperInfo array */
   printf ("typedef struct {\n"
-	  "  int name;\n"
-	  "  float width;\n"
+          "  int name;\n"
+          "  float width;\n"
           "  float height;\n"
-	  "  int display_name;\n"
-	  "  int ppd_name;\n"
-	  "} PaperInfo;\n\n"
-          "const PaperInfo _gtk_standard_names_offsets[] = {\n");
+          "  int display_name;\n"
+          "  int ppd_name;\n"
+          "} PaperInfo;\n\n"
+          "static const PaperInfo standard_names_offsets[] = {\n");
 
   for (i = 0; i < n_infos; i++)
     {
       width = height = 0.0;
       if (!parse_media_size (standard_names[i].size, &width, &height))
-	printf ("failed to parse size %s\n", standard_names[i].size);
+        printf ("failed to parse size %s\n", standard_names[i].size);
 
-      printf ("  { %4d, %g, %g, %4d, %4d },\n", 
-	      find_name (standard_names[i].name),
-	      width, height,
-	      find_name (standard_names[i].display_name),
-	      find_name (standard_names[i].ppd_name));
+      printf ("  { %4d, %g, %g, %4d, %4d },\n",
+              find_name (standard_names[i].name),
+              width, height,
+              find_name (standard_names[i].display_name),
+              find_name (standard_names[i].ppd_name));
     }
-  
+
   printf ("};\n\n");
 
 
   /* dump extras */
 
-  printf ("const struct {\n"
-	  "  int ppd_name;\n"
-	  "  int standard_name;\n"
-	  "} _gtk_extra_ppd_names_offsets[] = {\n");
+  printf ("static const struct {\n"
+          "  int ppd_name;\n"
+          "  int standard_name;\n"
+          "} extra_ppd_names_offsets[] = {\n");
 
   for (i = 0; i < n_extra; i++)
     {
-      printf ("  { %4d, %4d },\n", 
-	      find_name (extra_ppd_names[i].ppd_name),
-	      find_name (extra_ppd_names[i].standard_name));
+      printf ("  { %4d, %4d },\n",
+              find_name (extra_ppd_names[i].ppd_name),
+              find_name (extra_ppd_names[i].standard_name));
     }
-  
+
   printf ("};\n\n");
 
   return 0;
