@@ -401,14 +401,6 @@ typedef enum
   GTK_TOOLBAR_BOTH_HORIZ
 } GtkToolbarStyle;
 
-/* Data update types (for ranges) */
-typedef enum
-{
-  GTK_UPDATE_CONTINUOUS,
-  GTK_UPDATE_DISCONTINUOUS,
-  GTK_UPDATE_DELAYED
-} GtkUpdateType;
-
 /* Window position types */
 typedef enum
 {
@@ -581,6 +573,7 @@ typedef enum
 
 /**
  * GtkStateFlags:
+ * @GTK_STATE_FLAG_NORMAL: State during normal operation.
  * @GTK_STATE_FLAG_ACTIVE: Widget is active.
  * @GTK_STATE_FLAG_PRELIGHT: Widget has a mouse pointer over it.
  * @GTK_STATE_FLAG_SELECTED: Widget is selected.
@@ -592,6 +585,7 @@ typedef enum
  */
 typedef enum
 {
+  GTK_STATE_FLAG_NORMAL       = 0,
   GTK_STATE_FLAG_ACTIVE       = 1 << 0,
   GTK_STATE_FLAG_PRELIGHT     = 1 << 1,
   GTK_STATE_FLAG_SELECTED     = 1 << 2,
