@@ -46,7 +46,7 @@
 #include "gtkintl.h"
 #include "gtkprivate.h"
 #include "gtkstock.h"
-#include "gdkkeysyms.h"
+#include "gtktypebuiltins.h"
 
 /**
  * SECTION:gtkinfobar
@@ -157,10 +157,9 @@ static void     gtk_info_bar_get_property (GObject        *object,
                                            guint           prop_id,
                                            GValue         *value,
                                            GParamSpec     *pspec);
-static void     gtk_info_bar_style_set    (GtkWidget      *widget,
-                                           GtkStyle       *prev_style);
-static gboolean gtk_info_bar_expose       (GtkWidget      *widget,
-                                           GdkEventExpose *event);
+static void     gtk_info_bar_style_updated (GtkWidget      *widget);
+static gboolean gtk_info_bar_draw         (GtkWidget      *widget,
+                                           cairo_t        *cr);
 static void     gtk_info_bar_buildable_interface_init     (GtkBuildableIface *iface);
 static GObject *gtk_info_bar_buildable_get_internal_child (GtkBuildable  *buildable,
                                                            GtkBuilder    *builder,
@@ -296,40 +295,42 @@ gtk_info_bar_close (GtkInfoBar *info_bar)
 }
 
 static gboolean
-gtk_info_bar_expose (GtkWidget      *widget,
-                     GdkEventExpose *event)
+gtk_info_bar_draw (GtkWidget      *widget,
+                   cairo_t        *cr)
 {
   GtkInfoBarPrivate *priv = GTK_INFO_BAR (widget)->priv;
-  const char* type_detail[] = {
-    "infobar-info",
-    "infobar-warning",
-    "infobar-question",
-    "infobar-error",
-    "infobar"
+  const char* type_class[] = {
+    GTK_STYLE_CLASS_INFO,
+    GTK_STYLE_CLASS_WARNING,
+    GTK_STYLE_CLASS_QUESTION,
+    GTK_STYLE_CLASS_ERROR,
+    NULL
   };
 
   if (priv->message_type != GTK_MESSAGE_OTHER)
     {
-      GtkAllocation allocation;
-      const char *detail;
+      GtkStyleContext *context;
 
-      detail = type_detail[priv->message_type];
+      context = gtk_widget_get_style_context (widget);
 
-      gtk_widget_get_allocation (widget, &allocation);
+      gtk_style_context_save (context);
 
-      gtk_paint_box (gtk_widget_get_style (widget),
-                     gtk_widget_get_window (widget),
-                     GTK_STATE_NORMAL,
-                     GTK_SHADOW_OUT,
-                     NULL,
-                     widget,
-                     detail,
-                     allocation.x, allocation.y,
-                     allocation.width, allocation.height);
+      if (type_class[priv->message_type])
+        gtk_style_context_add_class (context,
+                                     type_class[priv->message_type]);
+
+      gtk_render_background (context, cr, 0, 0,
+                             gtk_widget_get_allocated_width (widget),
+                             gtk_widget_get_allocated_height (widget));
+      gtk_render_frame (context, cr, 0, 0,
+                        gtk_widget_get_allocated_width (widget),
+                        gtk_widget_get_allocated_height (widget));
+
+      gtk_style_context_restore (context);
     }
 
-  if (GTK_WIDGET_CLASS (gtk_info_bar_parent_class)->expose_event)
-    GTK_WIDGET_CLASS (gtk_info_bar_parent_class)->expose_event (widget, event);
+  if (GTK_WIDGET_CLASS (gtk_info_bar_parent_class)->draw)
+    GTK_WIDGET_CLASS (gtk_info_bar_parent_class)->draw (widget, cr);
 
   return FALSE;
 }
@@ -348,8 +349,8 @@ gtk_info_bar_class_init (GtkInfoBarClass *klass)
   object_class->set_property = gtk_info_bar_set_property;
   object_class->finalize = gtk_info_bar_finalize;
 
-  widget_class->style_set = gtk_info_bar_style_set;
-  widget_class->expose_event = gtk_info_bar_expose;
+  widget_class->style_updated = gtk_info_bar_style_updated;
+  widget_class->draw = gtk_info_bar_draw;
 
   klass->close = gtk_info_bar_close;
 
@@ -494,91 +495,7 @@ gtk_info_bar_class_init (GtkInfoBarClass *klass)
 }
 
 static void
-gtk_info_bar_update_colors (GtkInfoBar *info_bar)
-{
-  GtkWidget *widget = GTK_WIDGET (info_bar);
-  GtkInfoBarPrivate *priv = info_bar->priv;
-  GdkColor info_default_border_color     = { 0, 0xb800, 0xad00, 0x9d00 };
-  GdkColor info_default_fill_color       = { 0, 0xff00, 0xff00, 0xbf00 };
-  GdkColor warning_default_border_color  = { 0, 0xb000, 0x7a00, 0x2b00 };
-  GdkColor warning_default_fill_color    = { 0, 0xfc00, 0xaf00, 0x3e00 };
-  GdkColor question_default_border_color = { 0, 0x6200, 0x7b00, 0xd960 };
-  GdkColor question_default_fill_color   = { 0, 0x8c00, 0xb000, 0xd700 };
-  GdkColor error_default_border_color    = { 0, 0xa800, 0x2700, 0x2700 };
-  GdkColor error_default_fill_color      = { 0, 0xf000, 0x3800, 0x3800 };
-  GdkColor other_default_border_color    = { 0, 0xb800, 0xad00, 0x9d00 };
-  GdkColor other_default_fill_color      = { 0, 0xff00, 0xff00, 0xbf00 };
-  GdkColor *fg, *bg;
-  GdkColor sym_fg, sym_bg;
-  GtkStyle *style;
-  const char* fg_color_name[] = {
-    "info_fg_color",
-    "warning_fg_color",
-    "question_fg_color",
-    "error_fg_color",
-    "other_fg_color"
-  };
-  const char* bg_color_name[] = {
-    "info_bg_color",
-    "warning_bg_color",
-    "question_bg_color",
-    "error_bg_color",
-    "other_bg_color"
-  };
-
-  style = gtk_widget_get_style (widget);
-
-  if (gtk_style_lookup_color (style, fg_color_name[priv->message_type], &sym_fg) &&
-      gtk_style_lookup_color (style, bg_color_name[priv->message_type], &sym_bg))
-    {
-      fg = &sym_fg;
-      bg = &sym_bg;
-    }
-  else
-    {
-      switch (priv->message_type)
-        {
-        case GTK_MESSAGE_INFO:
-          fg = &info_default_border_color;
-          bg = &info_default_fill_color;
-          break;
-
-        case GTK_MESSAGE_WARNING:
-          fg = &warning_default_border_color;
-          bg = &warning_default_fill_color;
-          break;
-
-        case GTK_MESSAGE_QUESTION:
-          fg = &question_default_border_color;
-          bg = &question_default_fill_color;
-          break;
-
-        case GTK_MESSAGE_ERROR:
-          fg = &error_default_border_color;
-          bg = &error_default_fill_color;
-          break;
-
-        case GTK_MESSAGE_OTHER:
-          fg = &other_default_border_color;
-          bg = &other_default_fill_color;
-          break;
-
-        default:
-          g_assert_not_reached();
-          fg = NULL;
-          bg = NULL;
-        }
-    }
-
-  if (!gdk_color_equal (bg, &style->bg[GTK_STATE_NORMAL]))
-    gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, bg);
-  if (!gdk_color_equal (fg, &style->fg[GTK_STATE_NORMAL]))
-    gtk_widget_modify_fg (widget, GTK_STATE_NORMAL, fg);
-}
-
-static void
-gtk_info_bar_style_set (GtkWidget *widget,
-                        GtkStyle  *prev_style)
+gtk_info_bar_style_updated (GtkWidget *widget)
 {
   GtkInfoBar *info_bar = GTK_INFO_BAR (widget);
   gint button_spacing;
@@ -599,8 +516,6 @@ gtk_info_bar_style_set (GtkWidget *widget,
   gtk_box_set_spacing (GTK_BOX (info_bar->priv->content_area), content_area_spacing);
   gtk_container_set_border_width (GTK_CONTAINER (info_bar->priv->content_area),
                                   content_area_border);
-
-  gtk_info_bar_update_colors (info_bar);
 }
 
 static void
@@ -615,11 +530,11 @@ gtk_info_bar_init (GtkInfoBar *info_bar)
                                                 GTK_TYPE_INFO_BAR,
                                                 GtkInfoBarPrivate);
 
-  content_area = gtk_hbox_new (FALSE, 0);
+  content_area = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_widget_show (content_area);
   gtk_box_pack_start (GTK_BOX (info_bar), content_area, TRUE, TRUE, 0);
 
-  action_area = gtk_vbutton_box_new ();
+  action_area = gtk_button_box_new (GTK_ORIENTATION_VERTICAL);
   gtk_widget_show (action_area);
   gtk_button_box_set_layout (GTK_BUTTON_BOX (action_area), GTK_BUTTONBOX_END);
   gtk_box_pack_start (GTK_BOX (info_bar), action_area, FALSE, TRUE, 0);
@@ -739,7 +654,7 @@ gtk_info_bar_add_action_widget (GtkInfoBar *info_bar,
  *
  * Returns the action area of @info_bar.
  *
- * Returns: the action area.
+ * Returns: (transfer none): the action area
  *
  * Since: 2.18
  */
@@ -757,7 +672,7 @@ gtk_info_bar_get_action_area (GtkInfoBar *info_bar)
  *
  * Returns the content area of @info_bar.
  *
- * Returns: the content area.
+ * Returns: (transfer none): the content area
  *
  * Since: 2.18
  */
@@ -781,7 +696,7 @@ gtk_info_bar_get_content_area (GtkInfoBar *info_bar)
  * to the end of the info bars's action area. The button widget is
  * returned, but usually you don't need it.
  *
- * Returns: the button widget that was added
+ * Returns: (transfer none): the button widget that was added
  *
  * Since: 2.18
  */
@@ -1185,7 +1100,6 @@ gtk_info_bar_set_message_type (GtkInfoBar     *info_bar,
     {
       priv->message_type = message_type;
 
-      gtk_info_bar_update_colors (info_bar);
       gtk_widget_queue_draw (GTK_WIDGET (info_bar));
 
       atk_obj = gtk_widget_get_accessible (GTK_WIDGET (info_bar));
