@@ -19,13 +19,16 @@
  */
 
 #include "config.h"
+
 #include <string.h>
 #include <stdlib.h>
+
 #include <glib/gprintf.h>
-#include <gtk/gtk.h>
+
 #include "gtkprintsettings.h"
 #include "gtkprintutils.h"
-#include "gtkalias.h"
+#include "gtktypebuiltins.h"
+#include "gtkwidget.h"
 
 
 typedef struct _GtkPrintSettingsClass GtkPrintSettingsClass;
@@ -144,7 +147,7 @@ gtk_print_settings_copy (GtkPrintSettings *other)
  * 
  * Since: 2.10
  */
-const gchar *
+G_CONST_RETURN gchar *        
 gtk_print_settings_get (GtkPrintSettings *settings,
 			const gchar      *key)
 {
@@ -499,7 +502,7 @@ gtk_print_settings_foreach (GtkPrintSettings    *settings,
  *
  * Since: 2.10
  */
-const gchar *
+G_CONST_RETURN gchar *       
 gtk_print_settings_get_printer (GtkPrintSettings *settings)
 {
   return gtk_print_settings_get (settings, GTK_PRINT_SETTINGS_PRINTER);
@@ -1406,13 +1409,12 @@ gtk_print_settings_set_print_pages (GtkPrintSettings *settings,
 /**
  * gtk_print_settings_get_page_ranges:
  * @settings: a #GtkPrintSettings
- * @num_ranges: (out): return location for the length of the returned array
+ * @num_ranges: return location for the length of the returned array
  *
  * Gets the value of %GTK_PRINT_SETTINGS_PAGE_RANGES.
  *
- * Return value: (array length=num_ranges) (transfer full): an array
- *     of #GtkPageRange<!-- -->s.  Use g_free() to free the array when
- *     it is no longer needed.
+ * Return value: (transfer full): an array of #GtkPageRange<!-- -->s.
+ *     Use g_free() to free the array when it is no longer needed.
  *
  * Since: 2.10
  */
@@ -1469,7 +1471,7 @@ gtk_print_settings_get_page_ranges (GtkPrintSettings *settings,
 /**
  * gtk_print_settings_set_page_ranges:
  * @settings: a #GtkPrintSettings
- * @page_ranges: (array length=num_ranges): an array of #GtkPageRange<!-- -->s
+ * @page_ranges: an array of #GtkPageRange<!-- -->s
  * @num_ranges: the length of @page_ranges
  * 
  * Sets the value of %GTK_PRINT_SETTINGS_PAGE_RANGES.
@@ -1515,7 +1517,7 @@ gtk_print_settings_set_page_ranges  (GtkPrintSettings *settings,
  *
  * Since: 2.10
  */
-const gchar *
+G_CONST_RETURN gchar *
 gtk_print_settings_get_default_source (GtkPrintSettings *settings)
 {
   return gtk_print_settings_get (settings, GTK_PRINT_SETTINGS_DEFAULT_SOURCE);
@@ -1550,7 +1552,7 @@ gtk_print_settings_set_default_source (GtkPrintSettings *settings,
  *
  * Since: 2.10
  */
-const gchar *
+G_CONST_RETURN gchar *
 gtk_print_settings_get_media_type (GtkPrintSettings *settings)
 {
   return gtk_print_settings_get (settings, GTK_PRINT_SETTINGS_MEDIA_TYPE);
@@ -1585,7 +1587,7 @@ gtk_print_settings_set_media_type (GtkPrintSettings *settings,
  *
  * Since: 2.10
  */
-const gchar *
+G_CONST_RETURN gchar *
 gtk_print_settings_get_dither (GtkPrintSettings *settings)
 {
   return gtk_print_settings_get (settings, GTK_PRINT_SETTINGS_DITHER);
@@ -1649,7 +1651,7 @@ gtk_print_settings_set_finishings (GtkPrintSettings *settings,
  *
  * Since: 2.10
  */
-const gchar *
+G_CONST_RETURN gchar *
 gtk_print_settings_get_output_bin (GtkPrintSettings *settings)
 {
   return gtk_print_settings_get (settings, GTK_PRINT_SETTINGS_OUTPUT_BIN);
@@ -1922,7 +1924,3 @@ gtk_print_settings_to_key_file (GtkPrintSettings  *settings,
 			      (GtkPrintSettingsFunc) add_value_to_key_file,
 			      &data);
 }
-
-
-#define __GTK_PRINT_SETTINGS_C__
-#include "gtkaliasdef.c"
