@@ -31,6 +31,7 @@
 #include "gtkwin32embedwidget.h"
 #include "gtkintl.h"
 #include "gtkprivate.h"
+#include "gtkwindowprivate.h"
 
 
 static void            gtk_win32_embed_widget_realize               (GtkWidget        *widget);
@@ -95,12 +96,12 @@ _gtk_win32_embed_widget_new (GdkNativeWindow parent_id)
   embed_widget = g_object_new (GTK_TYPE_WIN32_EMBED_WIDGET, NULL);
   
   embed_widget->parent_window =
-    gdk_window_lookup_for_display (gdk_display_get_default (),
-				   parent_id);
+    gdk_win32_window_lookup_for_display (gdk_display_get_default (),
+					 parent_id);
   
   if (!embed_widget->parent_window)
     embed_widget->parent_window =
-      gdk_window_foreign_new_for_display (gdk_display_get_default (),
+      gdk_win32_window_foreign_new_for_display (gdk_display_get_default (),
 					  parent_id);
   
   return GTK_WIDGET (embed_widget);
@@ -149,7 +150,8 @@ gtk_win32_embed_widget_window_process (HWND hwnd, UINT msg, WPARAM wparam, LPARA
   GtkWin32EmbedWidget *embed_widget;
   gpointer user_data;
 
-  window = gdk_window_lookup ((GdkNativeWindow)hwnd);
+  window = gdk_win32_window_lookup_for_display (gdk_display_get_default (),
+						(GdkNativeWindow)hwnd);
   if (window == NULL) {
     g_warning ("No such window!");
     return 0;
@@ -243,14 +245,14 @@ gtk_win32_embed_widget_realize (GtkWidget *widget)
   styles = GetWindowLongPtr(GDK_WINDOW_HWND (gdk_window), GWL_STYLE);
   SetWindowLongPtrW(GDK_WINDOW_HWND (gdk_window), GWL_STYLE, styles | WS_TABSTOP);
 
-  gtk_widget_style_attach (widget);
-  gtk_style_set_background (gtk_widget_get_style (widget), gdk_window, GTK_STATE_NORMAL);
+  gtk_style_context_set_background (gtk_widget_get_style_context (widget),
+                                    gdk_window);
 }
 
 static void
 gtk_win32_embed_widget_show (GtkWidget *widget)
 {
-  GTK_WIDGET_SET_FLAGS (widget, GTK_VISIBLE);
+  gtk_widget_set_visible (widget, TRUE);
   
   gtk_widget_realize (widget);
   gtk_container_check_resize (GTK_CONTAINER (widget));
@@ -260,7 +262,7 @@ gtk_win32_embed_widget_show (GtkWidget *widget)
 static void
 gtk_win32_embed_widget_hide (GtkWidget *widget)
 {
-  GTK_WIDGET_UNSET_FLAGS (widget, GTK_VISIBLE);
+  gtk_widget_set_visible (widget, FALSE);
   gtk_widget_unmap (widget);
 }
 
