@@ -722,7 +722,6 @@ test_types (void)
     "  <object class=\"GtkHButtonBox\" id=\"hbuttonbox\"/>"
     "  <object class=\"GtkHBox\" id=\"hbox\"/>"
     "  <object class=\"GtkHPaned\" id=\"hpaned\"/>"
-    "  <object class=\"GtkHRuler\" id=\"hruler\"/>"
     "  <object class=\"GtkHScale\" id=\"hscale\"/>"
     "  <object class=\"GtkHScrollbar\" id=\"hscrollbar\"/>"
     "  <object class=\"GtkHSeparator\" id=\"hseparator\"/>"
@@ -749,7 +748,6 @@ test_types (void)
     "  <object class=\"GtkVScrollbar\" id=\"vscrollbar\"/>"
     "  <object class=\"GtkVSeparator\" id=\"vseparator\"/>"
     "  <object class=\"GtkViewport\" id=\"viewport\"/>"
-    "  <object class=\"GtkVRuler\" id=\"vruler\"/>"
     "  <object class=\"GtkVPaned\" id=\"vpaned\"/>"
     "  <object class=\"GtkVScale\" id=\"vscale\"/>"
     "  <object class=\"GtkWindow\" id=\"window\"/>"
@@ -1314,6 +1312,7 @@ test_combo_box (void)
   g_object_unref (builder);
 }
 
+#if 0
 static void
 test_combo_box_entry (void)
 {
@@ -1384,6 +1383,7 @@ test_combo_box_entry (void)
 
   g_object_unref (builder);
 }
+#endif
 
 static void
 test_cell_view (void)
@@ -2603,7 +2603,9 @@ main (int argc, char **argv)
   g_test_add_func ("/Builder/TreeView Column", test_treeview_column);
   g_test_add_func ("/Builder/IconView", test_icon_view);
   g_test_add_func ("/Builder/ComboBox", test_combo_box);
+#if 0
   g_test_add_func ("/Builder/ComboBox Entry", test_combo_box_entry);
+#endif
   g_test_add_func ("/Builder/CellView", test_cell_view);
   g_test_add_func ("/Builder/Dialog", test_dialog);
   g_test_add_func ("/Builder/Accelerators", test_accelerators);
