@@ -473,7 +473,7 @@ gail_expander_get_keybinding (AtkAction *action,
             guint key_val; 
 
             key_val = gtk_label_get_mnemonic_keyval (GTK_LABEL (label)); 
-            if (key_val != GDK_VoidSymbol)
+            if (key_val != GDK_KEY_VoidSymbol)
               return_value = gtk_accelerator_name (key_val, GDK_MOD1_MASK);
             g_free (expander->activate_keybinding);
             expander->activate_keybinding = return_value;
